import { App } from 'vue';
import { InjectionKey } from 'vue';

export declare type CheckboxInput = InputBase & {
    type: 'checkbox';
    value: boolean;
};

export declare type ColorInput = InputBase & {
    type: 'color';
    value: string;
};

export declare function createDynamicForms(options?: DynamicFormsOptions): DynamicFormsPlugin;

export declare type CustomInput = InputBase & {
    type: 'custom-field';
};

export declare interface DynamicFormsOptions {
    autoValidate?: boolean;
    form?: FormOptions;
}

export declare interface DynamicFormsPlugin {
    options?: DynamicFormsOptions;
    install(app: App): void;
}

export declare const dynamicFormsSymbol: InjectionKey<DynamicFormsPlugin>;

export declare const email: (control: FormControl<any>) => {
    email: boolean;
};

export declare type EmailInput = InputBase & {
    type: 'email';
    value: string;
};

export declare type FormControl<T = InputType> = T & {
    valid: boolean;
    invalid: boolean;
    dirty: boolean;
    touched: boolean;
    errors: ValidationErrors | null;
};

export declare interface FormOptions {
    customClass?: string;
    method?: string;
    netlify?: boolean;
    netlifyHoneypot?: string;
}

export declare interface FormValidation {
    validator: ValidatorFn;
    text: string;
}

export declare interface InputBase {
    value: any;
    name: string;
    label: string;
    disabled?: boolean;
    customClass?: string;
    placeholder?: string;
    validations?: FormValidation[];
}

export declare type InputType = TextInput | NumberInput | SelectInput | TextAreaInput | CheckboxInput | RadioInput | EmailInput | PasswordInput | RadioInput | ColorInput | UrlInput | CustomInput;

export declare type InputTypeKeys = 'text' | 'email' | 'url' | 'number' | 'password' | 'textarea' | 'select' | 'checkbox' | 'radio' | 'color';

export declare const isEmptyInputValue: (value: any) => boolean;

export declare const max: (max: number) => (control: FormControl<any>) => {
    max: {
        max: number;
        actual: any;
    };
};

export declare const maxLength: (maxLength: number) => (control: FormControl<any>) => {
    maxlength: {
        requiredLength: number;
        actualLength: any;
    };
};

export declare const min: (min: number) => (control: FormControl<any>) => {
    min: {
        min: number;
        actual: number;
    };
};

export declare const minLength: (minLength: number) => (control: FormControl<any>) => {
    minlength: {
        requiredLength: number;
        actualLength: any;
    };
};

export declare type NumberInput = InputBase & {
    min?: number;
    max?: number;
    step?: number;
    value: number;
    type: 'number';
};

export declare type PasswordInput = InputBase & {
    type: 'password';
    value: string;
};

export declare const pattern: (pattern: string) => (control: any) => {
    pattern: {
        requiredPattern: string | RegExp;
        actualValue: any;
    };
};

export declare type RadioInput = InputBase & {
    type: 'radio';
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
    value: string;
};

export declare const required: (control: FormControl<any>) => {
    required: boolean;
};

export declare type SelectInput<T = any> = InputBase & {
    type: 'select';
    value: T;
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
};

export declare type TextAreaInput = InputBase & {
    cols?: number;
    rows?: number;
    type: 'textarea';
    value: string;
};

export declare type TextInput = InputBase & {
    value: string;
    type: 'text';
};

export declare const url: (control: FormControl<any>) => {
    email: boolean;
};

export declare type UrlInput = InputBase & {
    type: 'url';
    value: string;
};

export declare function useDynamicForms(): DynamicFormsPlugin;

declare type ValidationErrors = {
    [key: string]: any;
};

declare interface ValidatorFn {
    (control: FormControl<any> | undefined): ValidationErrors | null;
}

export { }
